import * as grpc from '@grpc/grpc-js';
import * as protoLoader from '@grpc/proto-loader/';
import { SloggerProxyDefault } from '../browser/slogger-proxy';
import { injectable, inject } from 'inversify';
import { Slogger, IpcMessageType } from '../common/slogger';

var PROTO_PATH = __dirname + '/../../../backend/slogger-service/slogger_service_api/proto/slogger_service.proto';

@injectable()
export class SloggerElectronProxy extends SloggerProxyDefault {

    private SloggerService: any;
    @inject(Slogger) protected readonly logger: Slogger;

    constructor ()
    {
        super();
        this.SloggerService = (this.loadProtofile() as any).SLoggerService;
    }

    public importFile(proto: any) : Promise<unknown>{

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.importFile", {} );

        const client = this.createClient();

        let self = this;

        let promise = new Promise(function(resolve, reject) {

            client.importFile(
                proto, 
                (error: grpc.ServiceError, response: any) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.importFile", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.importFile", response);
                        resolve(response);
                    }
                    client.close();
                });
    
        });
        return promise;

    }

    public getDistinctFields(proto: any): Promise<unknown> {

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.getDistinctFields", proto );

        const client = this.createClient();

        let self = this;

        let promise = new Promise(function(resolve, reject) {

            client.getDistinctFields(
                proto, 
                (error: grpc.ServiceError, response: any) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.getDistinctFields", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.getDistinctFields", response);
                        resolve(response);
                    }
                    client.close();
                });
    
        });
        return promise;
        
    }

    public clearData(proto: any): Promise<unknown> {

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.clearData", proto );

        const client = this.createClient();

        let self = this;

        let promise = new Promise(function(resolve, reject) {

            client.clearData(
                proto, 
                (error: grpc.ServiceError, response: any) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.clearData", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.clearData", response);
                        resolve(response);
                    }
                    client.close();
                });
    
        });
        return promise;
        
    }


    public changeQuery(proto: any) : Promise<unknown>{

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.changeQuery", proto);

        const client = this.createClient();

        let self = this;

        let promise = new Promise(function(resolve, reject) {

            client.changeQuery(
                proto, 
                (error: grpc.ServiceError, response: any) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.changeQuery", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.changeQuery", response);
                        resolve(response);
                    }
                    client.close();
                });
    
        });
        return promise;

    }

    public getEventsFromTo(proto: any) : Promise<unknown>{

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.getEventsFromTo", proto );
        
        const client = this.createClient();

        let self = this;

        let promise = new Promise(function(resolve, reject) {

            client.getEventsFromTo(
                proto, 
                (error: grpc.ServiceError, response: any) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.getEventsFromTo", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.getEventsFromTo", {size: response.events.length});
                        resolve(response);
                    }
                    client.close();
                });
    
        });
        return promise;
    }

    public unregisterOnQueryChanged(proto: any) : Promise<unknown>{

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.unregisterOnQueryChanged", proto );
        
        const client = this.createClient();

        let self = this;

        let promise = new Promise(function(resolve, reject) {

            client.unregisterOnQueryChanged(
                proto, 
                (error: grpc.ServiceError, response: any) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.unregisterOnQueryChanged", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.unregisterOnQueryChanged", response);
                        resolve(response);
                    }
                    client.close();
                });
    
        });
        return promise;
    }

    public registerOnQueryChanged(proto: any, callback: any) {

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.registerOnQueryChanged", proto);

        const client = this.createClient();

        var call = client.registerOnQueryChanged(proto);

        let self = this;

        call.on('data', function(notification: any) {
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnQueryChanged", notification);
            callback(notification.resultSize);
        });

        call.on('error', function(error: grpc.ServiceError) {
            self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.registerOnQueryChanged", {error: error.message});
            callback(new Error(error.message));
            client.close();
        });

        call.on('end', function() {
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnQueryChanged", "STOPPED");
            client.close();
        });

    }

    public unregisterOnImportFile(proto: any) : Promise<unknown>{

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.unregisterOnImportFile", proto );
        
        const client = this.createClient();

        let self = this;

        let promise = new Promise(function(resolve, reject) {

            client.unregisterOnImportFile(
                proto, 
                (error: grpc.ServiceError, response: any) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.unregisterOnImportFile", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.unregisterOnImportFile", response);
                        resolve(response);
                    }
                    client.close();
                });
    
        });
        return promise;
    }

    public registerOnImportFile(proto: any, callback: any) {

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.registerOnImportFile", proto);

        const client = this.createClient();

        var call = client.registerOnImportFile(proto);

        let self = this;

        call.on('data', function(notification: any) {
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnImportFile", notification);
            callback(notification);
        });

        call.on('error', function(error: grpc.ServiceError) {
            self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.registerOnImportFile", {error: error.message});
            callback(new Error(error.message));
            client.close();
        });

        call.on('end', function() {
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnImportFile", "STOPPED");
            client.close();
        });

    }
    
    private loadProtofile(): grpc.GrpcObject {

        console.info("Load protobuf file from "+PROTO_PATH);

        const packageDefinition = protoLoader.loadSync(
            PROTO_PATH,
            {keepCase: true,
             longs: String,
             enums: String,
             defaults: true,
             oneofs: true
        });

        return grpc.loadPackageDefinition(packageDefinition);
    }

    private createClient(): any {
        return new this.SloggerService('localhost:50051', grpc.credentials.createInsecure());
    }

}